﻿using System;
using System.Net;
using System.Security;
using System.Windows.Threading;
using Microsoft.Devices;
using Microsoft.Devices.Sensors;

namespace MaciejGrabek.WP7Accelerometer
{
    public class WP7AccelerometerProvider
    {
        private string _dataUrl;
        private bool _dataUrlError = false;

        //future task - retry get data after some time...
        //private DateTime _dataUrlErrorDate = DateTime.MinValue;

        private Random _rand;
        private WebClient _client;

        private string[] dataSpliter = new string[] { ";" };

        private DispatcherTimer _timer;

        private Accelerometer _accelerometer;


        // Summary:
        // Creates a new instance of the MGrabek.WP7EmulatorAccelerometer.EmulatorAccelerometer object.
        [SecuritySafeCritical]
        public WP7AccelerometerProvider(string dataUrl = "http://localhost:9876/AccelerometerEmulatorProxy.svc/GetData", int refreshRate = 100)
        {
            //check if we are running on device or emulator
            if (Microsoft.Devices.Environment.DeviceType == DeviceType.Device)
            {
                _accelerometer = new Accelerometer();
                _accelerometer.ReadingChanged += new EventHandler<AccelerometerReadingEventArgs>(_accelerometer_ReadingChanged);
            }
            else
            {
                if (refreshRate < 33)
                {
                    refreshRate = 33;
                }

                this._dataUrl = dataUrl;
                this._dataUrlError = false;
                this._rand = new Random(DateTime.Now.Second * DateTime.Now.Millisecond);
                this._client = new WebClient();

                this._client.DownloadStringCompleted += new DownloadStringCompletedEventHandler(_client_DownloadStringCompleted);

                this._timer = new DispatcherTimer();
                this._timer.Interval = TimeSpan.FromMilliseconds(refreshRate);
                this._timer.Tick += delegate
                {
                    RefreshAccelerometerDataFromControlPanel();
                };
            }
        }

        // Summary:
        //     Gets the current state of the accelerometer. The value is a member of the
        //     Microsoft.Devices.Sensors.SensorState enumeration.
        //
        // Returns:
        //     Type Microsoft.Devices.Sensors.SensorState.
        public SensorState State
        {
            get
            {
                if (Microsoft.Devices.Environment.DeviceType == DeviceType.Device) { return _accelerometer.State; }
                else { return SensorState.Ready; }
            }
        }

        // Summary:
        //     Occurs when new data arrives from the accelerometer.
        public event EventHandler<WP7AccelerometerReadingEventArgs> ReadingChanged;

        // Summary:
        //     Releases the managed and unmanaged resources used by the Microsoft.Devices.Sensors.Accelerometer.
        [SecuritySafeCritical]
        public void Dispose()
        {
            this._client.CancelAsync();
            this._timer.Stop();
            if (Microsoft.Devices.Environment.DeviceType == DeviceType.Device)
            {
                _accelerometer.Stop();
            }
        }

        // Summary:
        //     Starts data acquisition from the accelerometer.
        [SecuritySafeCritical]
        public void Start()
        {
            if (Microsoft.Devices.Environment.DeviceType == DeviceType.Device)
            {
                try
                {
                    _accelerometer.Stop();
                }
                catch (Exception ex) { }

                _accelerometer.Start();
            }
            else
            {
                try
                {
                    this._timer.Stop();
                }
                catch (Exception ex) { }

                this._dataUrlError = false;
                this._timer.Start();
            }
        }

        // Summary:
        //     Stops data acquisition from the accelerometer.
        [SecuritySafeCritical]
        public void Stop()
        {
            if (Microsoft.Devices.Environment.DeviceType == DeviceType.Device)
            {
                _accelerometer.Stop();
            }
            else
            {
                this._timer.Stop();
            }
        }

        #region internal methods

        private void _accelerometer_ReadingChanged(object sender, AccelerometerReadingEventArgs e)
        {
            if(this.ReadingChanged!=null)
                this.ReadingChanged(this, new WP7AccelerometerReadingEventArgs(e.X, e.Y, e.Z, e.Timestamp));
        }

        private void RefreshAccelerometerDataFromControlPanel()
        {
            try
            {
                if (!this._dataUrlError)
                {
                    if (!this._client.IsBusy)
                        this._client.DownloadStringAsync(new Uri(String.Format("{0}?{1}", this._dataUrl, this._rand.NextDouble())), null);
                }
                else
                {
                    if (this.ReadingChanged != null)
                        ReadingChanged(this, GetRandomData());
                }
            }
            catch (Exception ex)
            {
                this._dataUrlError = true;
                if (this.ReadingChanged != null)
                    ReadingChanged(this, GetRandomData());
            }
        }

        private void _client_DownloadStringCompleted(object sender, DownloadStringCompletedEventArgs e)
        {
            if (e.Error == null)
            {
                WP7AccelerometerReadingEventArgs args = ReadDataFromString(e.Result);
                if (this.ReadingChanged != null)
                    ReadingChanged(this, args);
            }
            else
            {
                this._dataUrlError = true;
                if (this.ReadingChanged != null)
                    ReadingChanged(this, GetRandomData());
            }
        }

        private WP7AccelerometerReadingEventArgs ReadDataFromString(string data)
        {
            try
            {
                data = data.Trim().Replace("\n", "").Replace("\t", "");
                data = data.Substring(6, data.Length - 8);
                string[] items = data.Split(dataSpliter, StringSplitOptions.RemoveEmptyEntries);

                return new WP7AccelerometerReadingEventArgs(double.Parse(items[0], System.Globalization.CultureInfo.InvariantCulture),
                    double.Parse(items[1], System.Globalization.CultureInfo.InvariantCulture),
                    double.Parse(items[2], System.Globalization.CultureInfo.InvariantCulture),
                    DateTime.Now);
            }
            catch (Exception ex)
            {
                return GetRandomData();
            }
        }

        private WP7AccelerometerReadingEventArgs GetRandomData()
        {
            return new WP7AccelerometerReadingEventArgs(this._rand.NextDouble() * this._rand.Next(-1, 2)
                , this._rand.NextDouble() * this._rand.Next(-1, 2)
                , this._rand.NextDouble() * this._rand.Next(-1, 2)
                , new DateTimeOffset(DateTime.Now));
        }

        #endregion
    }
}
